﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Concurrency.TestTools.UnitTesting;
using System.Threading;
using System.Threading.Tasks;

namespace AlpacaProject.ControlFlow
{
    public class CancelingExample
    {

        [UnitTestMethod]
        public void UsingACancellationToken()
        {
            var cts = new CancellationTokenSource();
            var po = new ParallelOptions()
            {
                CancellationToken = cts.Token,
                MaxDegreeOfParallelism = System.Environment.ProcessorCount,
            };

            // Lets listen in to know when the token was cancelled.
            // These callbacks should NOT throw exceptions.
            var token = cts.Token;
            token.Register(() => Console.WriteLine("Token is being cancelled."));

            Parallel.Invoke(
                () =>
                {
                    Thread.Sleep(10);
                    Console.WriteLine("Cancelling operation via CancellationToken.Cancel...");
                    cts.Cancel();
                },
                () =>
                {
                    try
                    {
                        Thread.Sleep(1);
                        int[] nums = Enumerable.Range(0, 1000000).ToArray();
                        Parallel.ForEach(nums, po, (num) =>
                        {
                            double d = Math.Sqrt(num) * Math.Sqrt(num * num);
                        });
                        Console.WriteLine("Operation completed without being cancelled.");
                    }
                    catch (OperationCanceledException e)
                    {
                        Console.WriteLine(e.Message);
                        Assert.IsTrue(token.IsCancellationRequested);
                    }
                });
        }
    }
}
