﻿using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Concurrency.TestTools.UnitTesting;
using System.Threading;

namespace AlpacaProject
{
    public static class Fibonacci
    {

        /// <summary>Gets nth number in the Fibonacci number sequence.</summary>
        /// <param name="n"></param>
        /// <param name="recursive"></param>
        /// <returns></returns>
        public static int Get(int n, bool recursive = false)
        {
            if (recursive)
                return GetRecursive(n);
            else
                return GetIterative(n);
        }

        /// <summary>Gets nth number in the Fibonacci number sequence using a recursive algorithm.</summary>
        public static int GetRecursive(int n)
        {
            if (n <= 0)
                return 0;
            if (n == 1 || n == 2)
                return 1;
            return GetRecursive(n - 1) + GetRecursive(n - 2);
        }

        /// <summary>Gets nth number in the Fibonacci number sequence using an iterative algorithm.</summary>
        public static int GetIterative(int n)
        {
            int prev = -1;
            int result = 1;
            for (int i = 0; i <= n; i++)
            {
                int sum = result + prev;
                prev = result;
                result = sum;
            }
            return result;
        }

        /// <summary>
        /// Returns a never-ending series of Fibonacci numbers starting at the nth number
        /// in the Fibonacci sequence.
        /// </summary>
        /// <param name="n">The nth number in the Fibonacci sequence to start at.</param>
        /// <returns></returns>
        public static IEnumerable<int> GetSeries(int n = 0)
        {
            if (n < 0)
                throw new ArgumentOutOfRangeException("n", n, "Cannot be negative.");

            int prev = -1;
            int result = 1;

            // First, get us up to the nth number
            for (int i = 0; i <= n; i++)
            {
                int sum = result + prev;
                prev = result;
                result = sum;
            }

            // And then just keep spitting them out
            do
            {
                yield return result;

                // next...
                int sum = result + prev;
                prev = result;
                result = sum;
            } while (true);
        }

    }
}
