﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Concurrency.TestTools.UnitTesting;
using System.Threading;

namespace AlpacaProject
{
    public class FibonacciTests
    {

        static readonly int[] KnownValues = new[] {
            0, 1, 1, 2, 3
            , 5, 8, 13, 21, 34
            , 55, 89, 144, 233, 377
            , 610, 987, 1597, 2584, 4181, 6765
        };

        [UnitTestMethod]
        [TestArgs(false)]
        [TestArgs(true)]
        public void Fibonacci_Get(bool recursive)
        {
            Console.WriteLine("recursive={0}", recursive);
            for (int n = 0; n < KnownValues.Length; n++)
                Assert.AreEqual(KnownValues[n], Fibonacci.Get(n, recursive), "n={0}", n);
        }

        [UnitTestMethod]
        public void Fibonacci_GetSeries()
        {
            var fibSeries = Fibonacci.GetSeries(0).Take(KnownValues.Length).ToArray();
            for (int n = 0; n < KnownValues.Length; n++)
                Assert.AreEqual(KnownValues[n], fibSeries[n], "n={0}", n);
        }

        [UnitTestMethod]
        [TestArgs(30, false)]
        [TestArgs(30, true)]
        public void Fibonacci_DivisibilityBy11(int nCount, bool recursive)
        {
            // See: http://en.wikipedia.org/wiki/Fibonacci_number#Divisibility_by_11
            Console.WriteLine("nCount={0}", nCount);
            Console.WriteLine("recursive={0}", recursive);

            DateTime startTime = DateTime.Now;
            for (int n = 0; n < nCount; n++)
            {
                int lastK = n + 9;
                int nPlus6Fib = 0;
                int sumFibs = 0;
                for (int k = n; k <= lastK; k++)
                {
                    int fib = Fibonacci.Get(k, recursive);
                    sumFibs += fib;
                    if (k == n + 6)
                        nPlus6Fib = fib;
                }

                Assert.AreEqual(0, sumFibs % 11, "n={0}: The sum should be evenly disible by 11.", n);
                Assert.AreEqual(nPlus6Fib, sumFibs / 11, "n={0}: sum should be a multiple of 11 and Fib(n+6).", n);
            }

            var timeTaken = DateTime.Now - startTime;
            Console.WriteLine("Time taken: {0:s\\.ffffff}", timeTaken);
        }

        [UnitTestMethod]
        [TestArgs(30)]
        public void Fibonacci_DivisibilityBy11_UsingSeries(int nCount)
        {
            // See: http://en.wikipedia.org/wiki/Fibonacci_number#Divisibility_by_11
            Console.WriteLine("nCount={0}", nCount);

            DateTime startTime = DateTime.Now;
            for (int n = 0; n < nCount; n++)
            {
                var fibs = Fibonacci.GetSeries(n).Take(10).ToArray();
                int sumFibs = fibs.Sum();
                int nPlus6Fib = fibs[6];

                Assert.AreEqual(0, sumFibs % 11, "n={0}: The sum should be evenly disible by 11.", n);
                Assert.AreEqual(nPlus6Fib, sumFibs / 11, "n={0}: sum should be a multiple of 11 and Fib(n+6).", n);
            }

            var timeTaken = DateTime.Now - startTime;
            Console.WriteLine("Time taken: {0:s\\.ffffff}", timeTaken);
        }

        [UnitTestMethod]
        [TestArgs(30)]
        public void Fibonacci_DivisibilityBy11_UsingSeries2(int nCount)
        {
            // See: http://en.wikipedia.org/wiki/Fibonacci_number#Divisibility_by_11
            Console.WriteLine("nCount={0}", nCount);

            DateTime startTime = DateTime.Now;
            for (int n = 0; n < nCount; n++)
            {
                var fibs = Fibonacci.GetSeries(n).Take(10);
                int sumFibs = 0;
                int nPlus6Fib = 0;
                int i = 0;
                foreach (int fib in fibs)
                {
                    sumFibs += fib;
                    if (i++ == 6)
                        nPlus6Fib = fib;
                }

                Assert.AreEqual(0, sumFibs % 11, "n={0}: The sum should be evenly disible by 11.", n);
                Assert.AreEqual(nPlus6Fib, sumFibs / 11, "n={0}: sum should be a multiple of 11 and Fib(n+6).", n);
            }

            var timeTaken = DateTime.Now - startTime;
            Console.WriteLine("Time taken: {0:s\\.ffffff}", timeTaken);
        }

    }
}
