﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Concurrency.TestTools.UnitTesting;
using Microsoft.ParallelComputingPlatform.ParallelExtensions.Samples;
using System.Threading.Tasks;

namespace AlpacaProject.RayTracing
{
    /// <summary>
    /// This test class implements the strategies mentioned in the paper
    /// Patterns of Parallel Programming (PoPP) by Stephen Toub
    /// Section: TOO FINE-GRAINED, TOO COURSE GRAINED
    /// See the <see cref="FineVsCoarseGrainedRayTracer"/> class for the ray 
    /// tracer implementation.
    /// </summary>
    /// <remarks>
    /// http://www.microsoft.com/downloads/details.aspx?FamilyID=86b3d32b-ad26-4bb8-a3ae-c1637026c3ee&displaylang=en
    /// </remarks>
    public class PoPP_FineVsCoarseGrained
    {

        [UnitTestMethod]
        [TestArgs(-1)]
        [TestArgs(1)]
        [TestArgs(2)]
        [TestArgs(4)]
        [TestArgs(8)]
        public void NestedParallelFor(int maxDegreeOfParallelism)
        {
            Console.WriteLine("maxDegreeOfParallelism={0}", maxDegreeOfParallelism);
            Console.WriteLine("Environment.ProcessorCount={0}", Environment.ProcessorCount);

            RayTracerTest.ValidateRayTracerOutput((w, h) => new FineVsCoarseGrainedRayTracer(w, h) {
                MaxDegreeOfParallelism = maxDegreeOfParallelism,
                Method = FineVsCoarseGrainedRayTracer.RenderMethod.NestedParallelFor,
            });
        }

        [UnitTestMethod]
        [TestArgs(-1)]
        [TestArgs(1)]
        [TestArgs(2)]
        [TestArgs(4)]
        [TestArgs(8)]
        public void FlattenedLoop(int maxDegreeOfParallelism)
        {
            Console.WriteLine("maxDegreeOfParallelism={0}", maxDegreeOfParallelism);
            Console.WriteLine("Environment.ProcessorCount={0}", Environment.ProcessorCount);

            RayTracerTest.ValidateRayTracerOutput((w, h) => new FineVsCoarseGrainedRayTracer(w, h) {
                MaxDegreeOfParallelism = maxDegreeOfParallelism,
                Method = FineVsCoarseGrainedRayTracer.RenderMethod.FlattenedLoop,
            });
        }

        [UnitTestMethod]
        [TestArgs(-1)]
        [TestArgs(1)]
        [TestArgs(2)]
        [TestArgs(4)]
        [TestArgs(8)]
        public void Partitioned(int maxDegreeOfParallelism)
        {
            Console.WriteLine("maxDegreeOfParallelism={0}", maxDegreeOfParallelism);
            Console.WriteLine("Environment.ProcessorCount={0}", Environment.ProcessorCount);

            RayTracerTest.ValidateRayTracerOutput((w, h) => new FineVsCoarseGrainedRayTracer(w, h) {
                MaxDegreeOfParallelism = maxDegreeOfParallelism,
                Method = FineVsCoarseGrainedRayTracer.RenderMethod.Partitioned,
            });
        }

    }
}
