﻿//--------------------------------------------------------------------------
// 
//  Copyright (c) Microsoft Corporation.  All rights reserved. 
// 
//  File: Scene.cs
//
//--------------------------------------------------------------------------

using System.Collections.Generic;

namespace Microsoft.ParallelComputingPlatform.ParallelExtensions.Samples
{
    public class Scene
    {

        public static Scene CreateDefaultScene()
        {
            SceneObject[] things =  {
                new Sphere( new Vector(-0.5,1,1.5), 0.5, Surfaces.MatteShiny),
                new Sphere( new Vector(0,1,-0.25), 1, Surfaces.Shiny),
               new Plane( new Vector(0,1,0), 0, Surfaces.CheckerBoard)
            };
            Light[] lights = {
                new Light(new Vector(-2,2.5,0),new Color(.5,.45,.41)),
                new Light(new Vector(2,4.5,2), new Color(.99,.95,.8))
            };
            Camera camera = Camera.Create(new Vector(2.75, 2, 3.75), new Vector(-0.6, .5, 0));

            return new Scene(things, lights, camera);
        }



        public SceneObject[] Things;
        public Light[] Lights;
        public Camera Camera;

        public Scene(SceneObject[] things, Light[] lights, Camera camera) { Things = things; Lights = lights; Camera = camera; }

        public IEnumerable<ISect> Intersect(Ray r)
        {
            foreach (SceneObject obj in Things)
            {
                yield return obj.Intersect(r);
            }
        }
    }
}
